@@ -5,10 +5,12 @@ use warnings;
 no strict 'refs';
 no warnings 'uninitialized';
 
-use Date::Calc qw(Days_in_Month Decode_Day_of_Week Nth_Weekday_of_Month_Year);
+use Date::Calc qw(Add_Delta_Days Days_in_Month
+                  Decode_Day_of_Week
+                  Nth_Weekday_of_Month_Year);
 use DateTime;
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 sub new {
     my $class = shift;
@@ -42,18 +44,28 @@ sub parse_datetime {
             $self->{datetime}->set_month($bits[1]);
             $self->{datetime}->set_year($bits[2]);
 
+            $self->{modified} = 1;
+
             return $self->_return_dt_object;
         }
     } else {
         @{$self->{tokens}} = split ' ', $date_string;
     }
 
+    my ($dont_proceed1,
+        $dont_proceed2,
+        $dont_proceed3) = (0,0,0);
+
     for ($self->{index} = 0; $self->{index} < @{$self->{tokens}}; $self->{index}++) {
 
         print "$self->{tokens}->[$self->{index}]\n" if $DEBUG;
 
         $self->_init_data;
 
+        if ($self->{tokens}->[$self->{index}] =~ /^second$/i) {
+            $self->{modified} = 1;
+        }
+
         if ($self->{tokens}->[$self->{index}+2] =~ /^ago$/i) {
             $self->_ago;
         }
@@ -70,28 +82,60 @@ sub parse_datetime {
             $self->{datetime}->set_year($1);
         }
 
-        $self->_months;
+        OUTER1: foreach my $match qw(in this) {
+            foreach my $i qw(-1 0 1 2) {
+                if ($self->{tokens}->[$self->{index}+$i] =~ /$match/i) {
+                    $dont_proceed1 = 1;
+                    last OUTER1;
+                }
+            }
+        }
+
+        $self->_months unless $dont_proceed1;
 
         if ($self->{tokens}->[$self->{index}] =~ /^at$/i) {
             next;
         } elsif ($self->{tokens}->[$self->{index}] =~ /^(\d{1,2})(\:\d{2})?(am|pm)?|(noon|midnight)$/i) {
-            my $hour_token    = $1;
-            my $min_token     = $2;
-            my $timeframe     = $3;
-            my $noon_midnight = $4;
-
-            $self->_at($hour_token, $min_token, $timeframe, $noon_midnight);
+        OUTER2: foreach my $match qw(day in month) {
+                foreach my $i qw(-1 0 1 2) {
+                    if ($self->{tokens}->[$self->{index}+$i] =~ /^$match$/i) {
+                        $dont_proceed2 = 1;
+                        last OUTER2;
+                    }
+                }
+            }
+            unless ($dont_proceed2) {
+                my $hour_token    = $1;
+                my $min_token     = $2;
+                my $timeframe     = $3;
+                my $noon_midnight = $4;
+                $self->_at($hour_token, $min_token, $timeframe, $noon_midnight);
+            }
         }
 
         if ($self->{tokens}->[$self->{index}] =~ /^(\d{1,2})(?:st|nd|rd|th)? ?$/i) {
-            $self->_number($1);
+        OUTER3: foreach my $match qw(hours day week month) {
+                foreach my $i qw(-1 0 1 2) {
+                    if ($self->{tokens}->[$self->{index}+$i] =~ /$match/i) {
+                       $dont_proceed3 = 1;
+                       last OUTER3;
+                    }
+                }
+            }
+            foreach my $weekday (keys %{$self->{weekdays}}) {
+                if ($self->{tokens}->[$self->{index}+1] =~ /$weekday/i) {
+                    $dont_proceed3 = 1;
+                    last OUTER3;
+                }
+            }
+            $self->_number($1) unless $dont_proceed3;
         }
 
         if ($self->{tokens}->[$self->{index}] =~ /^\d{4}$/) {
             $self->{datetime}->set_year($self->{tokens}->[$self->{index}]);
         }
 
-        if ($self->{tokens}->[$self->{index}] !~ /^(?:this|next|last)$/i
+        if ($self->{tokens}->[$self->{index}]      !~ /^(?:this|next|last)$/i
             && $self->{tokens}->[$self->{index}-1] !~ /^(?:this|next|last)$/i
             && $self->{tokens}->[$self->{index}-2] !~ /^(?:this|next|last)$/i
             && $self->{tokens}->[$self->{index}+1] !~ /^(?:this|next|last)$/i) {
@@ -161,6 +205,8 @@ sub _return_dt_object {
     $self->{day}   = "0$self->{day}"   unless length($self->{day})   == 2;
     $self->{month} = "0$self->{month}" unless length($self->{month}) == 2;
 
+    $self->{modified} = 0;
+
     my $dt = DateTime->new(year   => $self->{year},
                            month  => $self->{month},
                            day    => $self->{day},
@@ -177,19 +223,19 @@ sub _ago {
 
     if ($new_tokens[1] =~ /^hour(?:s)?$/i) {
         $self->{datetime}->subtract(hours => $new_tokens[0]);
-        $self->_set_modified();
+        $self->_set_modified;
     } elsif ($new_tokens[1] =~ /^day(?:s)?$/i) {
         $self->{datetime}->subtract(days => $new_tokens[0]);
-        $self->_set_modified();
+        $self->_set_modified;
     } elsif ($new_tokens[1] =~ /^week(?:s)?$/i) {
         $self->{datetime}->subtract(days => (7 * $new_tokens[0]));
-        $self->_set_modified();
+        $self->_set_modified;
     } elsif ($new_tokens[1] =~ /^month(?:s)?$/i) {
         $self->{datetime}->subtract(months => $new_tokens[0]);
-        $self->_set_modified();
+        $self->_set_modified;
     } elsif ($new_tokens[1] =~ /^year(?:s)?$/i) {
         $self->{datetime}->subtract(years => $new_tokens[0]);
-        $self->_set_modified();
+        $self->_set_modified;
     }
 }
 
@@ -201,34 +247,34 @@ sub _now {
     if ($new_tokens[1] =~ /^day(?:s)?$/i) {
         if ($new_tokens[2] =~ /^before$/i) {
             $self->{datetime}->subtract(days => $new_tokens[0]);
-            $self->_set_modified();
+            $self->_set_modified;
         } elsif ($new_tokens[2] =~ /^from$/i) {
             $self->{datetime}->add(days => $new_tokens[0]);
-            $self->_set_modified();
+            $self->_set_modified;
         }
     } elsif ($new_tokens[1] =~ /^week(?:s)?$/i) {
         if ($new_tokens[2] =~ /^before$/i) {
             $self->{datetime}->subtract(days => (7 * $new_tokens[0]));
-            $self->_set_modified();
+            $self->_set_modified;
         } elsif ($new_tokens[2] =~ /^from$/i) {
             $self->{datetime}->add(days => (7 * $new_tokens[0]));
-            $self->_set_modified();
+            $self->_set_modified;
         }
      } elsif ($new_tokens[1] =~ /^month(?:s)?$/i) {
          if ($new_tokens[2] =~ /^before$/i) {
              $self->{datetime}->subtract(months => $new_tokens[0]);
-             $self->_set_modified();
+             $self->_set_modified;
          } elsif ($new_tokens[2] =~ /^from$/i) {
              $self->{datetime}->add(months => $new_tokens[0]);
-             $self->_set_modified();
+             $self->_set_modified;
          }
      } elsif ($new_tokens[1] =~ /^year(?:s)?$/i) {
          if ($new_tokens[2] =~ /^before$/i) {
              $self->{datetime}->subtract(years => $new_tokens[0]);
-             $self->_set_modified();
+             $self->_set_modified;
          } elsif ($new_tokens[2] =~ /^from$/i) {
              $self->{datetime}->add(years => $new_tokens[0]);
-             $self->_set_modified();
+             $self->_set_modified;
          }
      }
 }
@@ -245,15 +291,15 @@ sub _daytime {
     if ($self->{tokens}->[$self->{index}] =~ /^morning$/i) {
         $self->{datetime}->set_hour($hour_token ? $hour_token : '08' - $self->{hours_before});
         undef $self->{hours_before};
-        $self->_set_modified();
+        $self->_set_modified;
     } elsif ($self->{tokens}->[$self->{index}] =~ /^afternoon$/i) {
         $self->{datetime}->set_hour($hour_token ? $hour_token + 12 : '14' - $self->{hours_before});
         undef $self->{hours_before};
-        $self->_set_modified();
+        $self->_set_modified;
     } else {
         $self->{datetime}->set_hour( $hour_token ? $hour_token + 12 : '14' - $self->{hours_before});
         undef $self->{hours_before};
-        $self->_set_modified();
+        $self->_set_modified;
     }
 
     $self->{datetime}->set_minute(00);
@@ -264,16 +310,20 @@ sub _months {
 
     foreach my $key_month (keys %{$self->{months}}) {
         my $key_month_short = substr($key_month, 0, 3);
-        if ($self->{tokens}->[$self->{index}] =~ /$key_month/i
-            || $self->{tokens}->[$self->{index}] =~ /$key_month_short/i) {
-            $self->{datetime}->set_month($self->{months}->{$key_month});
-            $self->_set_modified();
-            if ($self->{tokens}->[$self->{index}+1] =~ /^(\d{1,2})(?:st|nd|rd|th)? ?$/i) {
-                $self->{datetime}->set_day($1);
-            } elsif ($self->{tokens}->[$self->{index}-1] =~ /^(\d{1,2})(?:st|nd|rd|th)? ?$/i) {
-                  $self->{datetime}->set_day($1);
-            }
-            splice(@{$self->{tokens}}, $self->{index}, 2);
+        foreach my $i qw(0 1) {
+            if ($self->{tokens}->[$self->{index}+$i] =~ /$key_month/i
+                || $self->{tokens}->[$self->{index}+$i] =~ /$key_month_short/i) {
+                   $self->{datetime}->set_month($self->{months}->{$key_month});
+                   $self->_set_modified;
+                   if ($self->{tokens}->[$self->{index}+$i+1] =~ /^(\d{1,2})(?:st|nd|rd|th)? ?$/i) {
+                       $self->{datetime}->set_day($1);
+                       splice(@{$self->{tokens}}, $self->{index}+$i+1, 1);
+                   } elsif ($self->{tokens}->[$self->{index}+$i-1] =~ /^(\d{1,2})(?:st|nd|rd|th)? ?$/i) {
+                      $self->{datetime}->set_day($1);
+                      splice(@{$self->{tokens}}, $self->{index}+$i-1, 1);
+
+                   }
+             }
         }
     }
 }
@@ -288,18 +338,18 @@ sub _number {
         if ($self->{datetime}->month() > 12) {
             $self->{datetime}->subtract(months => 12);
         }
-        $self->_set_modified();
+        $self->_set_modified;
     } elsif ($self->{tokens}->[$self->{index}+1] =~ /hour(?:s)/i) {
         if ($self->{tokens}->[$self->{index}+2] =~ /before/i) {
             $self->{hours_before} = $often;
-            $self->_set_modified();
+            $self->_set_modified;
         } elsif ($self->{tokens}->[$self->{index}+2] =~ /after/i) {
             $self->{hours_after} = $often;
-            $self->_set_modified();
+            $self->_set_modified;
         }
     } else {
         $self->{datetime}->set_day($often);
-        $self->_set_modified();
+        $self->_set_modified;
     }
 }
 
@@ -316,7 +366,7 @@ sub _at {
         $min_token =~ s!:!! if defined($min_token);
         $self->{datetime}->set_minute($min_token || 00);
 
-        $self->_set_modified();
+        $self->_set_modified;
 
         if ($timeframe) {
             if ($timeframe =~ /^pm$/i) {
@@ -327,7 +377,7 @@ sub _at {
             }
         }
     } elsif ($noon_midnight) {
-        $self->_set_modified();
+        $self->_set_modified;
         $self->{hours_before} ||= 0;
         if ($noon_midnight =~ /noon/i) {
             $self->{datetime}->set_hour(12);
@@ -358,7 +408,7 @@ sub _weekday {
                 $days_diff = $self->{datetime}->wday - $self->{weekdays}->{$key_weekday};
                 $self->{datetime}->subtract(days => $days_diff);
             }
-            $self->_set_modified();
+            $self->_set_modified;
             last;
         }
     }
@@ -367,14 +417,19 @@ sub _weekday {
 sub _this_in {
     my $self = shift;
 
+    if ($self->{tokens}->[$self->{index}+1] =~ /hour(?:s)/i) {
+        $self->{datetime}->add(hours => $self->{tokens}->[$self->{index}]);
+        $self->_set_modified;
+        return;
+    }
+
     foreach my $key_weekday (keys %{$self->{weekdays}}) {
         my $weekday_short = lc(substr($key_weekday,0,3));
-
         if ($self->{tokens}->[$self->{index}] =~ /$key_weekday/i || $self->{tokens}->[$self->{index}] eq $weekday_short) {
             my $days_diff = $self->{weekdays}->{$key_weekday} - $self->{datetime}->wday;
             $self->{datetime}->add(days => $days_diff);
             $self->{buffer} = '';
-            $self->_set_modified();
+            $self->_set_modified;
             last;
         }
 
@@ -383,13 +438,12 @@ sub _this_in {
             my $days_diff = Decode_Day_of_Week($weekday) - $self->{datetime}->wday;
             $self->{datetime}->add(days => $days_diff);
             $self->{buffer} = '';
-            $self->_set_modified();
+            $self->_set_modified;
             last;
         }
 
         foreach my $month (keys %{$self->{months}}) {
-            if ($self->{tokens}->[$self->{index}] =~ /$month/i) {
-
+             if ($self->{tokens}->[$self->{index}] =~ /$month/i) {
                 foreach my $weekday (keys %{$self->{weekdays}}) {
                     if ($self->{tokens}->[$self->{index}-2] =~ /$weekday/i) {
 
@@ -401,7 +455,8 @@ sub _this_in {
                         $self->{datetime}->set_month($month);
                         $self->{datetime}->set_day($day);
 
-                        $self->_set_modified();
+                        $self->_set_modified;
+                        splice(@{$self->{tokens}}, $self->{index}-3, 4);
                     }
                 }
             }
@@ -419,7 +474,7 @@ sub _next {
             my $days_diff = (7 - $self->{datetime}->wday) + Decode_Day_of_Week($key_weekday);
             $self->{datetime}->add(days => $days_diff);
             $self->{buffer} = '';
-            $self->_set_modified();
+            $self->_set_modified;
             last;
         }
 
@@ -428,19 +483,33 @@ sub _next {
             my $days_diff = (7 - $self->{datetime}->wday) + Decode_Day_of_Week($weekday);
             $self->{datetime}->add(days => $days_diff);
             $self->{buffer} = '';
-            $self->_set_modified();
+            $self->_set_modified;
             last;
         }
 
         if ($self->{tokens}->[$self->{index}] =~ /^month$/i) {
             $self->{datetime}->add(months => 1);
-            $self->_set_modified();
+            if ($self->{tokens}->[$self->{index}-2] =~ /^day$/i) {
+                my $day = $self->{tokens}->[$self->{index}-3];
+                $day =~ s/^(\d{1,2})(?:st|nd|rd|th)$/$1/i;
+                $self->{datetime}->set_day($day);
+            }
+            $self->_setmonthday;
+            $self->{buffer} = '';
+            $self->_set_modified;
             last;
         }
 
         if ($self->{tokens}->[$self->{index}] =~ /^year$/i) {
             $self->{datetime}->add(years => 1);
-            $self->_set_modified();
+            if ($self->{tokens}->[$self->{index}-2] =~ /^month$/i) {
+                my $month = $self->{tokens}->[$self->{index}-3];
+                $month =~ s/^(\d{1,2})(?:st|nd|rd|th)$/$1/i;
+                $self->{datetime}->set_month($month);
+            }
+            $self->_setyearday;
+            $self->{buffer} = '';
+            $self->_set_modified;
             last;
         }
     }
@@ -456,37 +525,45 @@ sub _last {
             my $days_diff = $self->{datetime}->wday + (7 - $self->{weekdays}->{$key_weekday});
             $self->{datetime}->subtract(days => $days_diff);
             $self->{buffer} = '';
-            $self->_set_modified();
+            $self->_set_modified;
             last;
         }
     }
 
     if ($self->{tokens}->[$self->{index}] =~ /^week$/i) {
-
         if (exists $self->{weekdays}->{ucfirst(lc($self->{tokens}->[$self->{index}+1]))}) {
-            my $weekday = ucfirst(lc($self->{tokens}->[$self->{index}+1]));
-            my $days_diff = $self->{datetime}->wday + (7 - $self->{weekdays}->{$weekday});
-            $self->{datetime}->subtract(days => $days_diff);
-            $self->{buffer} = '';
-            $self->_set_modified();
+            $self->_setweekday($self->{index}+1);
         } elsif (exists $self->{weekdays}->{ucfirst(lc($self->{tokens}->[$self->{index}-2]))}) {
-            my $weekday = ucfirst(lc($self->{tokens}->[$self->{index}-2]));
-            my $days_diff = $self->{datetime}->wday + (7 - $self->{weekdays}->{$weekday});
+            $self->_setweekday($self->{index}-2);
+        } elsif ($self->{tokens}->[$self->{index}-2] =~ /^day$/i) {
+            my $days_diff = (7 + $self->{datetime}->wday);
             $self->{datetime}->subtract(days => $days_diff);
+            my $day = $self->{tokens}->[$self->{index}-3];
+            $day =~ s/^(\d{1,2})(?:st|nd|rd|th)$/$1/i;
+            $self->{datetime}->add(days => $day);
             $self->{buffer} = '';
-            $self->_set_modified();
+            $self->_set_modified;
         }
     }
 
     if ($self->{tokens}->[$self->{index}] =~ /^month$/i) {
         $self->{datetime}->subtract(months => 1);
-        $self->_set_modified();
+        $self->_set_modified;
+        if ($self->{tokens}->[$self->{index}-2] =~ /^day$/i) {
+            my $day = $self->{tokens}->[$self->{index}-3];
+            $day =~ s/^(\d{1,2})(?:st|nd|rd|th)$/$1/i;
+            $self->{datetime}->set_day($day);
+        }
+        $self->{buffer} = '';
+        $self->_setmonthday;
     }
 
     if ($self->{tokens}->[$self->{index}] =~ /^year$/i) {
         $self->{datetime}->subtract(years => 1);
-        $self->_set_modified();
-    }
+        $self->_setyearday;
+        $self->_set_modified;
+        $self->{buffer} = '';
+     }
 }
 
 sub _monthdays_limit {
@@ -497,12 +574,12 @@ sub _monthdays_limit {
     if ($self->{datetime}->day > $monthdays) {
         $self->{datetime}->add(months => 1);
         $self->{datetime}->set_day($self->{datetime}->day - $monthdays);
-        $self->_set_modified();
+        $self->_set_modified;
     } elsif ($self->{datetime}->day < 1) {
         $monthdays = Days_in_Month($self->{datetime}->year, ($self->{datetime}->month-1));
         $self->{datetime}->subtract(months => 1);
         $self->{datetime}->set_day($monthdays - $self->{datetime}->day);
-        $self->_set_modified();
+        $self->_set_modified;
     }
 }
 
@@ -517,7 +594,7 @@ sub _day {
             $self->{datetime}->add(days => 1);
         }
 
-        $self->_set_modified();
+        $self->_set_modified;
 
         if ($self->{hours_before}) {
             $self->{datetime}->set_hour(24 - $self->{hours_before});
@@ -533,6 +610,46 @@ sub _day {
     }
 }
 
+sub _setyearday {
+    my $self = shift;
+
+    if ($self->{tokens}->[$self->{index}-2] =~ /^day$/i) {
+        $self->{datetime}->set_month(1);
+        my $days = $self->{tokens}->[$self->{index}-3];
+        $days =~ s/^(\d{1,3})(?:st|nd|rd|th)$/$1/i;
+        my ($year, $month, $day) = Add_Delta_Days($self->{datetime}->year, 1, 1, $days - 1);
+        $self->{datetime}->set_day($day);
+        $self->{datetime}->set_month($month);
+        $self->{datetime}->set_year($year);
+    }
+}
+
+sub _setmonthday {
+    my $self = shift;
+
+    foreach my $weekday (keys %{$self->{weekdays}}) {
+        if ($self->{tokens}->[$self->{index}-2] =~ /$weekday/i) {
+            my ($often) = $self->{tokens}->[$self->{index}-3] =~ /^(\d{1,2})(?:st|nd|rd|th)?$/i;
+            my ($year, $month, $day) =
+                Nth_Weekday_of_Month_Year($self->{datetime}->year, ($self->{datetime}->month),
+                                          $self->{weekdays}->{$weekday}, $often);
+            $self->{datetime}->set_year($year);
+            $self->{datetime}->set_month($month);
+            $self->{datetime}->set_day($day);
+        }
+    }
+}
+
+sub _setweekday {
+    my ($self, $index) = @_;
+
+    my $weekday = ucfirst(lc($self->{tokens}->[$index]));
+    my $days_diff = $self->{datetime}->wday + (7 - $self->{weekdays}->{$weekday});
+    $self->{datetime}->subtract(days => $days_diff);
+    $self->{buffer} = '';
+    $self->_set_modified;
+}
+
 1;
 __END__
 
@@ -543,9 +660,9 @@ DateTime::Format::Natural::EN - Create machine readable date/time with natural p
 =head1 SYNOPSIS
 
  use DateTime::Format::Natural::EN;
- 
+
  $parse = DateTime::Format::Natural::EN->new();
- 
+
  $dt = $parse->parse_datetime($date_string);
 
 =head1 DESCRIPTION
@@ -619,7 +736,6 @@ Below are some examples of human readable date/time input:
  3rd wednesday in november
  3rd month next year
  7 hours before tomorrow at noon
- 3 months ago saturday at 5:00pm
 
 =head2 Specific Dates
 
